@@ -54,10 +54,10 @@ scale_x_units <- function(..., guide = ggplot2::waiver(), position = "bottom",
   sc <- ggplot2::continuous_scale(
     c("x", "xmin", "xmax", "xend", "xintercept", "xmin_final", "xmax_final",
       "xlower", "xmiddle", "xupper"),
-    "position_c", identity, ...,
+    palette = identity, ...,
     guide = guide,
     position = position,
-    super = MakeScaleContinuousPositionUnits()
+    super = make_scale_units()
   )
   sc$units <- as_units(unit)
   set_sec_axis(sec.axis, sc)
@@ -73,19 +73,19 @@ scale_y_units <- function(..., guide = ggplot2::waiver(), position = "left",
   sc <- ggplot2::continuous_scale(
     c("y", "ymin", "ymax", "yend", "yintercept", "ymin_final", "ymax_final",
       "lower", "middle", "upper"),
-    "position_c", identity, ...,
+    palette = identity, ...,
     guide = guide,
     position = position,
-    super = MakeScaleContinuousPositionUnits()
+    super = make_scale_units()
   )
   sc$units <- as_units(unit)
   set_sec_axis(sec.axis, sc)
 }
 
-MakeScaleContinuousPositionUnits <- function() {
+make_scale_units <- function(parent=ggplot2::ScaleContinuousPosition) {
   ggplot2::ggproto(
     "ScaleContinuousPositionUnits",
-    ggplot2::ScaleContinuousPosition,
+    parent,
     units = NULL,
 
     map = function(self, x, limits = self$get_limits()) {
@@ -95,23 +95,34 @@ MakeScaleContinuousPositionUnits <- function() {
         else units(x) <- as_units(1, self$units)
         x <- drop_units(x)
       }
-      ggplot2::ggproto_parent(
-        ggplot2::ScaleContinuousPosition, self)$map(x, limits)
+      ggplot2::ggproto_parent(parent, self)$map(x, limits)
     },
 
     transform = function(self, x) {
       if (!is.null(self$units))
         units(x) <- as_units(1, self$units)
 
-      new_x <- ggplot2::ggproto_parent(
-        ggplot2::ScaleContinuousPosition, self)$transform(drop_units(x))
+      if (inherits(self$limits, "units")) {
+        units(self$limits) <- units(x)
+        self$limits <- drop_units(self$limits)
+      }
+
+      new_x <- ggplot2::ggproto_parent(parent, self)$transform(drop_units(x))
       as_units(new_x, units(x))
     },
 
-    make_title = function(self, title) {
-      if (!is.null(title))
-        title <- make_unit_label(title, as_units(1, self$units))
-      title
+    make_title = function(self, guide_title, scale_title, label_title) {
+      if (missing(label_title)) { # ggplot2 <= 3.5.1
+        title <- guide_title
+        if (!is.null(title))
+          title <- make_unit_label(title, as_units(1, self$units))
+        return(title)
+      }
+
+      if (!is.null(label_title))
+        label_title <- make_unit_label(label_title, as_units(1, self$units))
+      ggplot2::ggproto_parent(parent, self)$make_title(
+        guide_title, scale_title, label_title)
     }
   )
 }
@@ -135,3 +146,18 @@ scale_type.units <- function(x) {
          "  Please, attach it using 'library(units)' to properly show scales with units.")
   c("units", "continuous")
 }
+
+utils::globalVariables("caller_env")
+
+# registered in .onLoad()
+limits.units <- function(lims, var, call = caller_env())  {
+  if (length(lims) != 2)
+    stop("`", var, "` must be a two-element vector", call.=FALSE)
+
+  trans <- if (!any(is.na(lims)) && lims[1] > lims[2])
+    "reverse" else "identity"
+  name <- paste0("scale_", var, "_units")
+  sc <- match.fun(name)(limits=lims, transform=trans)
+  sc$call <- if (!is.null(call)) call else str2lang(paste0(name, "()"))
+  sc
+}
